# HG changeset patch
# User Laurence Rowe <laurence@lrowe.co.uk>
# Date 1278888848 -3600
# Node ID bf4d14f5143695587ca97d58feb6ad88aeec4f3f
# Parent  aed89f1bb357e289086f9a768e2e9958c4076b7c
Skip transform when there is no content (e.g. a proxied redirect)

diff --git a/src/http/modules/ngx_http_xslt_filter_module.c b/src/http/modules/ngx_http_xslt_filter_module.c
--- a/src/http/modules/ngx_http_xslt_filter_module.c
+++ b/src/http/modules/ngx_http_xslt_filter_module.c
@@ -249,6 +249,10 @@
         return ngx_http_next_body_filter(r, in);
     }
 
+    if (r->headers_out.content_length_n == 0) {
+        return ngx_http_xslt_send(r, ctx, in->buf);
+    }
+
     for (cl = in; cl; cl = cl->next) {
 
         if (ngx_http_xslt_add_chunk(r, ctx, cl->buf) != NGX_OK) {
# HG changeset patch
# User Laurence Rowe <laurence@lrowe.co.uk>
# Date 1331326885 0
# Node ID 9487b3a0e3ffef66ca46666419f69ac2cc45c552
# Parent  bf4d14f5143695587ca97d58feb6ad88aeec4f3f
Use xmlCtxtUseOptions to set options.

diff --git a/src/http/modules/ngx_http_xslt_filter_module.c b/src/http/modules/ngx_http_xslt_filter_module.c
--- a/src/http/modules/ngx_http_xslt_filter_module.c
+++ b/src/http/modules/ngx_http_xslt_filter_module.c
@@ -366,15 +366,14 @@
                           "xmlCreatePushParserCtxt() failed");
             return NGX_ERROR;
         }
+        xmlCtxtUseOptions(ctxt, XML_PARSE_NOENT|XML_PARSE_DTDLOAD
+                          |XML_PARSE_NOWARNING);
 
         ctxt->sax->externalSubset = ngx_http_xslt_sax_external_subset;
         ctxt->sax->setDocumentLocator = NULL;
-        ctxt->sax->warning = NULL;
         ctxt->sax->error = ngx_http_xslt_sax_error;
         ctxt->sax->fatalError = ngx_http_xslt_sax_error;
         ctxt->sax->_private = ctx;
-        ctxt->replaceEntities = 1;
-        ctxt->loadsubset = 1;
 
         ctx->ctxt = ctxt;
         ctx->request = r;
# HG changeset patch
# User Laurence Rowe <laurence@lrowe.co.uk>
# Date 1331327298 0
# Node ID c8349ca87381fb12d76546f4bba25197b17f2d13
# Parent  9487b3a0e3ffef66ca46666419f69ac2cc45c552
XML_PARSE_COMPACT to save memory

diff --git a/src/http/modules/ngx_http_xslt_filter_module.c b/src/http/modules/ngx_http_xslt_filter_module.c
--- a/src/http/modules/ngx_http_xslt_filter_module.c
+++ b/src/http/modules/ngx_http_xslt_filter_module.c
@@ -367,7 +367,7 @@
             return NGX_ERROR;
         }
         xmlCtxtUseOptions(ctxt, XML_PARSE_NOENT|XML_PARSE_DTDLOAD
-                          |XML_PARSE_NOWARNING);
+                          |XML_PARSE_NOWARNING|XML_PARSE_COMPACT);
 
         ctxt->sax->externalSubset = ngx_http_xslt_sax_external_subset;
         ctxt->sax->setDocumentLocator = NULL;
# HG changeset patch
# User Laurence Rowe <laurence@lrowe.co.uk>
# Date 1331329666 0
# Node ID 7145bd8cc1e214f4c3e58a0655eb6c8d006ed888
# Parent  c8349ca87381fb12d76546f4bba25197b17f2d13
xslt_html_parser

diff --git a/src/http/modules/ngx_http_xslt_filter_module.c b/src/http/modules/ngx_http_xslt_filter_module.c
--- a/src/http/modules/ngx_http_xslt_filter_module.c
+++ b/src/http/modules/ngx_http_xslt_filter_module.c
@@ -10,6 +10,7 @@
 #include <ngx_http.h>
 
 #include <libxml/parser.h>
+#include <libxml/HTMLparser.h>
 #include <libxml/tree.h>
 #include <libxslt/xslt.h>
 #include <libxslt/xsltInternals.h>
@@ -58,6 +59,7 @@
     ngx_hash_t                 types;
     ngx_array_t               *types_keys;
     ngx_array_t               *params;       /* ngx_http_xslt_param_t */
+    ngx_flag_t                 html_parser;
 } ngx_http_xslt_filter_loc_conf_t;
 
 
@@ -67,6 +69,7 @@
     xsltTransformContextPtr    transform;
     ngx_http_request_t        *request;
     ngx_array_t                params;
+    ngx_flag_t                 html_parser;
 
     ngx_uint_t                 done;         /* unsigned  done:1; */
 } ngx_http_xslt_filter_ctx_t;
@@ -150,6 +153,13 @@
       offsetof(ngx_http_xslt_filter_loc_conf_t, types_keys),
       &ngx_http_xslt_default_types[0] },
 
+    { ngx_string("xslt_html_parser"),
+      NGX_HTTP_LOC_CONF|NGX_CONF_FLAG,
+      ngx_conf_set_flag_slot,
+      NGX_HTTP_LOC_CONF_OFFSET,
+      offsetof(ngx_http_xslt_filter_loc_conf_t, html_parser),
+      NULL },
+
       ngx_null_command
 };
 
@@ -225,6 +235,8 @@
 
     r->main_filter_need_in_memory = 1;
 
+    ctx->html_parser = conf->html_parser;
+
     return NGX_OK;
 }
 
@@ -265,7 +277,11 @@
                 xmlFreeDoc(ctx->ctxt->myDoc);
             }
 
-            xmlFreeParserCtxt(ctx->ctxt);
+            if (ctx->html_parser) {
+                htmlFreeParserCtxt(ctx->ctxt);
+            } else {
+                xmlFreeParserCtxt(ctx->ctxt);
+            }
 
             return ngx_http_xslt_send(r, ctx, NULL);
         }
@@ -280,9 +296,13 @@
 
             wellFormed = ctx->ctxt->wellFormed;
 
-            xmlFreeParserCtxt(ctx->ctxt);
+            if (ctx->html_parser) {
+                htmlFreeParserCtxt(ctx->ctxt);
+            } else {
+                xmlFreeParserCtxt(ctx->ctxt);
+            }
 
-            if (wellFormed) {
+            if (wellFormed || ctx->html_parser) {
                 return ngx_http_xslt_send(r, ctx,
                                        ngx_http_xslt_apply_stylesheet(r, ctx));
             }
@@ -356,41 +376,76 @@
     ngx_buf_t *b)
 {
     int               err;
-    xmlParserCtxtPtr  ctxt;
+    xmlParserCtxtPtr  ctxt = NULL;
+    xmlCharEncoding   charset = XML_CHAR_ENCODING_NONE;
 
     if (ctx->ctxt == NULL) {
 
-        ctxt = xmlCreatePushParserCtxt(NULL, NULL, NULL, 0, NULL);
-        if (ctxt == NULL) {
-            ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
-                          "xmlCreatePushParserCtxt() failed");
-            return NGX_ERROR;
+        if (ctx->html_parser) {
+            if (r->headers_out.charset.len == 5
+                && ngx_strncasecmp(r->headers_out.charset.data,
+                                   (u_char *) "utf-8", 5) == 0)
+            {
+                charset = XML_CHAR_ENCODING_UTF8;
+            }
+
+            ctxt = htmlCreatePushParserCtxt(NULL, NULL, NULL, 0, NULL,
+                                            charset);
+            if (ctxt == NULL) {
+                ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
+                              "htmlCreatePushParserCtxt() failed");
+                return NGX_ERROR;
+            }
+
+            htmlCtxtUseOptions(ctxt, HTML_PARSE_RECOVER|HTML_PARSE_NOERROR
+                               |HTML_PARSE_NOWARNING|HTML_PARSE_COMPACT);
+
+        } else {
+            ctxt = xmlCreatePushParserCtxt(NULL, NULL, NULL, 0, NULL);
+            if (ctxt == NULL) {
+                ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
+                              "xmlCreatePushParserCtxt() failed");
+                return NGX_ERROR;
+            }
+
+            xmlCtxtUseOptions(ctxt, XML_PARSE_NOENT|XML_PARSE_DTDLOAD
+                              |XML_PARSE_NOWARNING|XML_PARSE_COMPACT);
+            ctxt->sax->externalSubset = ngx_http_xslt_sax_external_subset;
+            ctxt->sax->error = ngx_http_xslt_sax_error;
+            ctxt->sax->fatalError = ngx_http_xslt_sax_error;
         }
-        xmlCtxtUseOptions(ctxt, XML_PARSE_NOENT|XML_PARSE_DTDLOAD
-                          |XML_PARSE_NOWARNING|XML_PARSE_COMPACT);
 
-        ctxt->sax->externalSubset = ngx_http_xslt_sax_external_subset;
         ctxt->sax->setDocumentLocator = NULL;
-        ctxt->sax->error = ngx_http_xslt_sax_error;
-        ctxt->sax->fatalError = ngx_http_xslt_sax_error;
         ctxt->sax->_private = ctx;
 
         ctx->ctxt = ctxt;
         ctx->request = r;
     }
 
-    err = xmlParseChunk(ctx->ctxt, (char *) b->pos, (int) (b->last - b->pos),
-                        (b->last_buf) || (b->last_in_chain));
+    if (ctx->html_parser) {
+        err = htmlParseChunk(ctx->ctxt, (char *) b->pos,
+                             (int) (b->last - b->pos),
+                             (b->last_buf) || (b->last_in_chain));
 
-    if (err == 0) {
+        /* HTML_PARSE_RECOVER is on so errors are recoverable */
         b->pos = b->last;
         return NGX_OK;
+
+    } else {
+        err = xmlParseChunk(ctx->ctxt, (char *) b->pos,
+                            (int) (b->last - b->pos),
+                            (b->last_buf) || (b->last_in_chain));
+
+        if (err == 0) {
+            b->pos = b->last;
+            return NGX_OK;
+        }
+
+        ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
+                      "xmlParseChunk() failed, error:%d", err);
+
+        return NGX_ERROR;
     }
-
-    ngx_log_error(NGX_LOG_ERR, r->connection->log, 0,
-                  "xmlParseChunk() failed, error:%d", err);
-
-    return NGX_ERROR;
 }
 
 
@@ -1061,6 +1116,8 @@
      *     conf->params = NULL;
      */
 
+    conf->html_parser = NGX_CONF_UNSET;
+
     return conf;
 }
 
@@ -1083,6 +1140,8 @@
         conf->params = prev->params;
     }
 
+    ngx_conf_merge_value(conf->html_parser, prev->html_parser, 0);
+
     if (ngx_http_merge_types(cf, &conf->types_keys, &conf->types,
                              &prev->types_keys, &prev->types,
                              ngx_http_xslt_default_types)